/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.api;

import java.util.Map;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;

import org.ow2.petals.component.framework.api.su.ServiceUnitExtensionsValidator;
import org.ow2.petals.component.framework.api.su.ServiceUnitListener;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

/**
 * 
 * Created on 15 mai 2008
 * 
 * @author Christophe HAMERLING - EBM WebSourcing
 * @since 1.0
 * 
 */
public interface Component extends javax.jbi.component.Component, ComponentLifeCycle {

    /**
     * Returns the component delivery channel, used to send messages to petals
     * container
     * 
     * @return the delivery channel. Not null.
     */
    public DeliveryChannel getChannel();

    /**
     * The component context
     * 
     * @return the component context. Not null.
     */
    public ComponentContext getContext();

    /**
     * Returns null by default as this class does not provides JMX extension
     * MBean
     * 
     * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName();

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Component#getLifeCycle()
     */
    public ComponentLifeCycle getLifeCycle();

    /**
     * Returns the component logger
     * 
     * @return component logger. Not null.
     */
    public Logger getLogger();

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.Component#getServiceDescription(javax.jbi.servicedesc
     * .ServiceEndpoint)
     */
    public Document getServiceDescription(final ServiceEndpoint endpoint);

    public ServiceUnitListener getServiceUnitListener();

    /**
     * Get the Service Unit validator.
     * 
     * @return the {@code ServiceUnitExtensionsValidator}
     * 
     * @deprecated Define constraints in the component JBI descriptor schema instead.
     */
    public ServiceUnitExtensionsValidator getServiceUnitExtensionsValidator();

    /**
     * Get all the interceptors
     * 
     * @return
     */
    public Map<String, Interceptor> getInterceptors();

    /**
     * Get the interceptor that have the given name.
     * 
     * @param name
     * @return null if the interceptor is not found of is there are no
     *  interceptor in the component
     */
    public Interceptor getInterceptor(final String name);

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Component#getServiceUnitManager()
     */
    public javax.jbi.component.ServiceUnitManager getServiceUnitManager();

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.
     * ComponentContext)
     */
    public void init(final ComponentContext compContext) throws JBIException;

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.component.Component#isExchangeWithConsumerOkay(javax.jbi.
     * servicedesc.ServiceEndpoint, javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithConsumerOkay(final ServiceEndpoint endpoint,
            final MessageExchange exchange);

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.component.Component#isExchangeWithProviderOkay(javax.jbi.
     * servicedesc.ServiceEndpoint, javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithProviderOkay(final ServiceEndpoint endpoint,
            final MessageExchange exchange);

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.component.Component#resolveEndpointReference(org.w3c.dom.
     * DocumentFragment)
     */
    public ServiceEndpoint resolveEndpointReference(final DocumentFragment epr);

    public void setServiceUnitListener(final ServiceUnitListener serviceUnitListener);

    /**
     * 
     * @param serviceUnitExtensionsValidator
     * @deprecated
     */
    public void setServiceUnitExtensionsValidator(
            final ServiceUnitExtensionsValidator serviceUnitExtensionsValidator);

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#shutDown()
     */
    public void shutDown() throws JBIException;

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#start()
     */
    public void start() throws JBIException;

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#stop()
     */
    public void stop() throws JBIException;

    // public ComponentConfiguration getComponentConfiguration();

    /**
     * 
     * @return
     */
    // public JBIAcceptorManager getListenerManager();
    
    /**
     * Create a performance notification which the type RECEIVING_FROM_OUTSIDE
     * 
     * @return The performance notification to send.
     */
    // public PerformanceNotification
    // createPerformanceNotificationReceivingFromOutside(
    // final String userData, final Exchange msgExchange) throws JBIException;
    /**
     * Create a performance notification which the type SENDING_TO_OUTSIDE
     * 
     * @return The performance notification to send.
     */
    // public PerformanceNotification
    // createPerformanceNotificationSendingToOutside(
    // final String userData, final Exchange msgExchange) throws JBIException;
    /**
     * Create a performance notification which the type RECEIVING_FROM_NMR
     * 
     * @return The performance notification to send.
     */
    // public PerformanceNotification
    // createPerformanceNotificationReceivingFromNMR(
    // final String userData, final Exchange msgExchange) throws JBIException;
    /**
     * Create a performance notification which the type SENDING_TO_NMR
     * 
     * @return The performance notification to send.
     */
    // public PerformanceNotification createPerformanceNotificationSendingToNMR(
    // final String userData,
    // final Exchange msgExchange) throws JBIException;
    /**
     * Send a performance notification using the performance notifier
     * 
     * @param performanceNotification
     */
    // public void sendPerformanceNotification(final PerformanceNotification
    // performanceNotification) throws JBIException;
}
